/*******************************************************************************
 
 
 	    FILE:	CourseReader.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Abstract Factory to create CourseReaders which target various platforms.
				
 	RELATION:	CourseReader is a base abstract factory that is extended by
 				SFCourseReader, XMLCourseReader, and DBMSCourseReader classes 
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

 
package cscie247.asn3;

import java.util.*;
import java.io.*;


public abstract class CourseReader {
	
	protected	Calendar	pCal;						// calendar obj
	protected	ArrayList	pCoursesList;
	protected	String		pCourseReadErrors;			// list of all read 
														// errors thrown by parser

	// A holiday list will always be needed to exclude from course dates.
	protected	ArrayList	pHolidays;					// holiday list type Date
	
	
	// properties used for building ScheduleData objects
	protected	String		pCNumStr;					// course number
	protected	String		pTitleStr;					// course title
	protected	Date		pStartDate;					// start date
	protected	Date		pEndDate;					// end date
	protected	ArrayList	pOnDaysLst;					// list of days. Sun = 0
	
	
	/***************************************************************************
    *
    *	Constructor: CourseReader 
	*
	*	Initializes fields, most notably the pCourseReadErrors string used for 
	*	nonfatal errors
    */
	public CourseReader() {
		pCal = new GregorianCalendar();
		pCoursesList		= new ArrayList();	// default to an empty ArrayList
												//  [of type ScheduleData]
		pHolidays			= new ArrayList();	// default to an empty ArrayList 
												//  [of type Date]
		pCourseReadErrors	= "";				// No errors (yet).
	}
	
	
	/***************************************************************************
    *
    *	Empty method declarations
	*
    */
    protected abstract ArrayList readCourses(String mString) 
    											throws CriticalSDError;
	protected abstract void parseLine(String mLine) 
												throws ReadFileError;
	
	
	/***************************************************************************
    * 
    *	Mutator.  sets pHolidays
	*
	*		Allows the holiday Date list to be accessible to the CourseParser
	*
    *	@param		new value for pHolidays
    */
	protected void setHolidays(ScheduleData mHolidayData) {
        pHolidays = mHolidayData.getDates();
	}
	
	
	/***************************************************************************
    *
    *	buildCourse
  	*
  	*		Uses object parameters assigned by parseLine() to create a new 
  	*		ScheduleData object.  If bad data is encountered then that data
  	*		is simply omitted.
	*
	*	@return	Returns a new ScheduleData object.			
    */
	protected ScheduleData buildCourse() {
	
		// make a new ScheduleData
		ScheduleData mObjReturn = new ScheduleData(pCNumStr + ": ", pTitleStr);		
		
		Date	mTheDate	= pStartDate;			// date holder
		int		mDayOfWeek;							// holds the day of the week
		
		pCal.setTime(mTheDate);						// update calendar to mTheDate
						
		// while mTheDate <= pEndDate
		while( mTheDate.before(pEndDate) || mTheDate.equals(pEndDate) ) {
			
			// get the day of the week
			mDayOfWeek = pCal.get(Calendar.DAY_OF_WEEK) - 1;	
			
			if (isDateOnDaysList(mDayOfWeek) &&
			  ! isDateAHoliday(mTheDate))
				mObjReturn.addDate(mTheDate);		// add date to new ScheduleData
			
			pCal.add(Calendar.DATE, 1);				// go to next calendar day
			mTheDate = pCal.getTime();				// update mTheDate
		}
		return mObjReturn;		
	}
	

	/***************************************************************************
    *
    *	isDateOnDaysList
  	*
  	*		Returns true if [mDayOfWeek] is in the list [pOnDaysLst]
	*
	*	@param		mDayOfWeek	Day of week being tested for, 0 = Sunday.
	*	@return		true, if [mDayOfWeek] is in the parameter list pOnDaysLst			
    */
	protected boolean isDateOnDaysList(int mDayOfWeek) {
	
		boolean mReturn = false;
		
		// is date is in pOnDaysLst list?
		Iterator e = pOnDaysLst.iterator();
		while(e.hasNext() && ! mReturn) {
			if ( stringToInt((String) e.next()) == mDayOfWeek)
				mReturn = true;
		}
		
		return 	mReturn;	
	}
	

	/***************************************************************************
    *
    *	isDateAHoliday
  	*
  	*		Returns true if Date [mTheDate] is in the holiday list [pHolidays].
	*
	*	@param		mTheDate	Day of week being tested for, 0 = Sunday.
	*	@return		true, if [mTheDate] is in the holiday list [pHolidays].			
    */
	protected boolean isDateAHoliday(Date mTheDate) {
	
		boolean mReturn = false;
	
		// is date a holiday?
		Iterator e = pHolidays.iterator();
		while(e.hasNext() && ! mReturn) {
			if (mTheDate.equals( (Date) e.next()))
				mReturn = true;
		}
		
		return mReturn;	
	}
	
	
	/***************************************************************************
    *
    *	parseDaysToList
  	*
  	*		Converts the string [mOnDaysStr] which should be in format "Mon:Tues:Wed",
  	*		into an ArrayList list in format ["1","2","3"], where Sunday = 0.
  	*		ArrayList does not like integers, so I stored the numbers as
  	*			strings, and must convert them using stringToInt() when required.
	*
	*	@param		mOnDaysStr	String in "Mon:Tues:Wed" format.
	*
	*	@return		corresponding ArrayList list in format ["1","2","3"].				
    */
	protected ArrayList parseDaysToList(String mOnDaysStr) {
	
		ArrayList mReturn = new ArrayList();
		StringTokenizer dayDayDay = new StringTokenizer(mOnDaysStr, ":");
		String mDay;
		
		while(dayDayDay.hasMoreTokens()) {
		
			mDay = dayDayDay.nextToken();			// get the next token
			mDay.trim();							// trim out junk
			mDay = mDay.substring(0, 2);			// look for first two chars
			
			if (mDay.equalsIgnoreCase("Su")) {
				mReturn.add("0");
			} else if (mDay.equalsIgnoreCase("Mo")) {
				mReturn.add("1");
			} else if (mDay.equalsIgnoreCase("Tu")) {
				mReturn.add("2");
			} else if (mDay.equalsIgnoreCase("We")) {
				mReturn.add("3");
			} else if (mDay.equalsIgnoreCase("Th")) {
				mReturn.add("4");
			} else if (mDay.equalsIgnoreCase("Fr")) {
				mReturn.add("5");
			} else if (mDay.equalsIgnoreCase("Sa")) {
				mReturn.add("6");
			}
		}
			
		return mReturn;
		
	}
	
	
	/***************************************************************************
    *
    *	parseStrToDate
  	*
  	*		Converts a string of form "mm/dd/yy" into a Date object.
  	*		Unsucessful attempts result in null.
  	*
  	*	@param		The date string in neeed of conversion.
  	*	@return		A Date object, or null
    */
	protected Date parseStrToDate(String mDateStr) {
	
		Date mReturn = null;						// default return
		
		// ensure three values
		StringTokenizer mmddyyyy = new StringTokenizer(mDateStr, "/");
		if (mmddyyyy.countTokens() == 3) {
		
			int mMonth = stringToInt(mmddyyyy.nextToken());		
			int mDay   = stringToInt(mmddyyyy.nextToken());
			int mYear  = stringToInt(mmddyyyy.nextToken());
	
			// ensure valid month and day values
			if (mMonth >= 1 && mMonth <= 12 &&
				mDay   >= 1 && mDay   <= 31) {
				
				// month [mMonth] is from 0 to 11
				pCal = new GregorianCalendar(mYear, mMonth - 1, mDay);
				mReturn = pCal.getTime();
			}
		}
	
		return mReturn;
	}
	

	/***************************************************************************
    *
    *	stringToInt
    *
    *		Converts a string to an integer.
  	*
  	*	@param		The string to convert.
  	*	@return		An integer representation of the given string.
    */
	protected static int stringToInt(String stringObject) {
		return Integer.parseInt(stringObject.trim());
	}


	/***************************************************************************
    * 
    *	Accessor for the pCourseReadErrors String
	*
    *	@return		all read errors encountered by this FileReader instance.
    */
	public String getCourseReadErrors() {
		return pCourseReadErrors;
	}
	
	
	/***************************************************************************
    * 
    *	Accessor for the pCourseReadErrors ArrayList
	*
    *	@return		iterator to pCoursesList.
    */
	public Iterator getCoursesList() {
		return pCoursesList.iterator();
	}
	
	
	/***************************************************************************
    * 
    *	Accessor for the pHolidays ArrayList
	*
    *	@return		iterator to pHolidays.
    */
	public Iterator getHolidays() {
		return pHolidays.iterator();
	}
}