/*******************************************************************************
 
 
 	    FILE:	Distress.java
  
 	  AUTHOR:	Russell Lowke
					built to a specification by Bill Robinson 
					(see "Solution for Assignment 2", November 7th 2003)
     
 	   CLASS:	CSCIE-247, Assignment Four, 

 
 	 PURPOSE:	Distress is the focal point of interaction with external 
                objects.  It has three main functions: providing the ability for
                new users to gain access to the system, allowing existing users 
                to login to the system, and supporting ad hoc searches for 
                properties or mortgages.   There should be only one Distress 
                object instance.
				
ASSOCIATIONS:	The Distress object keeps a collection of User objects.
 	
	MODIFIED:	November 28th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress;

import java.util.*;

public class Distress {

	private static Distress pDistress = new Distress();				// singleton
	
    private static ArrayList pUsers = new ArrayList();		 // User objects in Distress				
    
    
    /**																		   |
    *
    *	Don't let anyone else instantiate this class   (singleton)
    *
    */
	private Distress() { }
    
    	
	/**																		   |
    * 
    *	This method takes all the attributes given and causes a new User to be 
    *	insterted in the system.  TypeUser is an enumeration of the kinds of 
    *	users (Realtor, Consumer, Lender).  It returns a reference to the new 
    *	user.  This throws an exception if loginName is already in use.
    *
	*	@param	TypeUser	The type of user registering
	*	@param	String		name of the user
	*	@param	String		address of user
	*	@param	String		city state and zipcode
	*	@param	String		user login name
	*	@param	String		user pin
	*	@param	String		user email
	*	@param	String		user credit card company
	*	@param	String		user credit card number
	*	@param	Object		object specific to the userType [mUt]
	*	@return	User		Object reference to the new User
    */
	public static User register (TypeUser  mUserType,
						  String    mName,
						  String	mAddress,
						  String	mCityStateZip,
						  String	mLoginName,
						  String	mPin,
						  String	mEmail,
						  String	mCreditCardCo,
						  String	mCreditCardNumber,
						  Object	mObject) throws ErrDistress, ErrRegistration {
		
		User mUser = null;
		
        // check for any Users with the same login name
		if ( getUser(mLoginName) != null )
			throw new ErrRegistration("A user already exists with a loginName of \"" 
								+ mLoginName + "\"\n");
		
		if (mUserType == TypeUser.CONSUMER) {
		
			// convert the mObject
			TypePurchasingTF mPurchasingTimeFrame = 
							(TypePurchasingTF) mObject;
			
			mUser = new Consumer(mUserType,
								 mName,
								 mAddress,
								 mCityStateZip,
								 mLoginName,
								 mPin, 
								 mEmail, 
								 mCreditCardCo, 
								 mCreditCardNumber,
								 mPurchasingTimeFrame);
			
		} else if (mUserType == TypeUser.LENDER) {
				
			// convert the mObject
			String mCompanyName = (String) mObject;
			
			mUser = new Lender(mUserType,
							   mName,
							   mAddress,
							   mCityStateZip,
							   mLoginName,
							   mPin, 
							   mEmail, 
							   mCreditCardCo, 
							   mCreditCardNumber,
							   mCompanyName);
					
		} else if (mUserType == TypeUser.REALTOR) {			
		
			// convert the mObject
			String mAgency = (String) mObject;
			
			mUser = new Realtor(mUserType,
								mName,
								mAddress,
								mCityStateZip,
								mLoginName,
								mPin, 
								mEmail, 
								mCreditCardCo, 
								mCreditCardNumber,
								mAgency);
		}
		
		if (mUser != null) {
			pDistress.add(mUser);
		}
		
		return mUser;
	}
	
    
    
	/**																		   |
    * 
    *	Causes this User to be removed from the system along with all their 
    *	appendages (for instance, when a Realtor is unregistered, it is 
    *	ultimately deleted from the system along with all Property objects it 
    *	owns).  Throws an exception if loginName doesn't exist.
    *
	*	@param	String	login name of the user to be removed
    */
    public static void unregister(String mLoginName) 
                                        throws ErrDistress, ErrRegistration {
		pDistress.remove(getUser(mLoginName));
    }
    
    
	/**																		   |
    * 
    *	Validates credentials and creates an object for a user.  Throws an
    *	exception if the credentials are invalid.
    *
	*	@param	String	name of the user
    *	@param	String	password entered by user
	*	@return	User	User object to that person
    */
    public static User login(String mLoginName, String mPassword) 
                                        throws ErrDistress, ErrRegistration {

        User mUser = getUser(mLoginName);
		if (mUser == null)
			throw new ErrRegistration("Could not find user with login of \n" + 
									mLoginName + "\n");
		
		if ( ! (mUser.getPin()).equals(mPassword) )
			throw new ErrRegistration("Pin/Password Incorect.");
		
        return mUser;
    }


	/**																		   |
    * 
    *	Returns a collection (possibly empty) of the Mortgage objects that match
    *	the critera.  Throws an exception if the serach criteria is empty.
    *
	*	@param	SearchCriteria	object which defines what to search for
	*	@return	ArrayList	 	list of Mortgage objects that match the search
    */
    public static ArrayList findMortgage(SearchCriteria mSearchCriteria) {
        ArrayList mReturn = new ArrayList();
        
		MortgageOffering  mMortgage;
		
		// iterate through MortgageOfferings looking for search critera match
		Iterator mItr = pDistress.listMortgageOfferings();
		while(mItr.hasNext()) {
			mMortgage = (MortgageOffering) mItr.next();
			
			// add any Mortgages that match search criteria to mReturn
			if ( mMortgage.match(mSearchCriteria) )
				mReturn.add(mMortgage);
		}
		
        return mReturn;
    }
    
    
	/**																		   |
    * 
    *	Returns a collection (possibly empty) of the Property objects that match
    *	the critera.  In searching for a Property, the proprty type and city 
    *	must be given.
    *
	*	@param	SearchCriteria	object which defines what to search for
	*	@return	ArrayList	 	list of Property objects that match the search
    */
    public static ArrayList findProperty(SearchCriteria mSearchCriteria)
                                                            throws ErrDistress {
	
		if ( ! mSearchCriteria.getHasPropertyTypeAndCity() ) {
			throw new ErrDistress("SearchCriteria requires a propertyType" +
									" and cityStateZip to find a property");
		}
		
        ArrayList mReturn = new ArrayList();
        
		Property  mProperty;
		
		// iterate through properties looking for search critera
		Iterator mItr = pDistress.listProperties();
		while(mItr.hasNext()) {
			mProperty = (Property) mItr.next();
			
			// add any properties that match search criteria to mReturn
			if ( mProperty.match(mSearchCriteria) )
				mReturn.add(mProperty);
		}
		
        return mReturn;
    }
    

	/**																		   |
    * 
    *	Returns the User object which has a specific loginID  [mLoginID]
	*
    *	@parame	String			the specific loginID
    *	@return	Iterator		Iterator to pMortgageOfferings.
    */
	public static User getUser(String mLoginID) {
	
		User	 mReturn = null;
		User     mUser;
		Iterator mItr = pDistress.list();
		
		while(mItr.hasNext() && mReturn == null) {
			mUser = (User) mItr.next();
			if ( (mUser.getLoginName()).equals(mLoginID) )
				mReturn = mUser;
		}
		
		return mReturn;
	}
    
    
	/**																		   |
    * 
    *	Adds the User instance mUser to the pUsers list, if not already there
	*
    *	@param	User		the User instance to be added to pUsers list.
    */
    public static void add(User mUser) throws ErrDistress, ErrRegistration {
    
        // check for other users with the same login name.
        User mOtherUser = getUser(  mUser.getLoginName()  );
		if (mOtherUser != null)
          throw new ErrRegistration("There is already a user with a login of \n"
									+ mUser.getLoginName() + "\n");
                                    
                                    
        // Note: error checking for any Users with the same login name
        // is done in register()

        Iterator mItr = findUser(mUser);
        
        if (mItr == null)
            pUsers.add(mUser);
        else
			throw new ErrRegistration("CAN'T ADD USER. \"" + mUser.getName() +
								  "\" is already in User List.");
			
    }
    
    
	/**																		   |
    * 
    *	Removes the User instance mUser from the pUsers list, if found.
	*
    *	@param	User		the User instance to be removed from pUsers list.
    */
    public static void remove(User mUser) throws ErrRegistration {
    
        Iterator mItr = findUser(mUser);
        
        if (mItr == null)
			throw new ErrRegistration("CAN'T REMOVE USER. \"" + mUser.getName() +
								  "\" is not in User List.");
        else
            mItr.remove();
    }
    
    
	/**																		   |
    * 
    *	Returns an iterator to the pUsers list
	*
    *	@return	String		a summary of pertinent values.
    */
    public static Iterator list() {
        return  pUsers.iterator();
    }
    
    
	/**																		   |
    * 
    *	Searches through the pUsers list looking for a User instance with the
    *	same reference as mUser.  If found, an Iterator is returned resting on
    *	the list item (so it may be removed easily), 
    *	otherwise a null is returned.
	*
    *	@return	Iterator	points to item mUser in pUsers. if mUser can't be
    *						found then returns null
    */
	private static Iterator findUser(User mUser) {
    
        boolean mFoundIt = false;
        Iterator mItr = list();
		while(mItr.hasNext() && ! mFoundIt)
            if (mUser == mItr.next())
                mFoundIt = true;
                
        if (! mFoundIt)
            mItr = null;
        
        return mItr;
    }
	
    
	/**																		   |
    * 
    *	Helper methods that invoke listHeldByUser(), thus returning an Iterator 
    *	a list of the requested objects
    *	
    *	@return	Iterator	to list of items held by Users of approp. type
    */
    public Iterator listProperties() {
		return listHeldByUser(TypeUser.REALTOR);
	}
	public Iterator listMortgageOfferings() {
		return listHeldByUser(TypeUser.LENDER);
	}
	public Iterator listPropertyProfiles() {
		return listHeldByUser(TypeUser.CONSUMER);
	}
    
    
	/**																		   |
    * 
    *	Helper method that returns an Iterator to a list of all
    *	Users of the type [mUserType]
    *	
    *	@param	TypeUser	type of User targeted
    *	@return	Iterator	list of objects held by that User
    */
	public Iterator listHeldByUser(TypeUser mUserType) {
		ArrayList mReturn = new ArrayList();
		
		// iterate through Users
		Iterator mItr = list();
		User	 mUser;
		Iterator mItr2;
		
		while(mItr.hasNext()) {
			mUser = (User) mItr.next();
			
			// find Realtors
			if (mUser.getUserType() == mUserType) {
				
				// add Realtors Properties to mReturn
				mItr2 = mUser.list();
				while(mItr2.hasNext())
					mReturn.add(mItr2.next());
			}
		}

		return mReturn.iterator();
	}
    
	
	/* 																		   |
	*			Accessors & Mutators
	*/
	public static Distress getDistress() {							// read only
		return pDistress;
	}
	
	public ArrayList getUsers() {
		return pUsers;
	}
	public void setUsers(ArrayList mUsers) {
		pUsers = mUsers;
	}
}