/*******************************************************************************
 
 
 	    FILE:	PropertyImpl.java
  
 	  AUTHOR:	Russell Lowke
					
 	   CLASS:	CSCIE-247, Final Project

 
 	 PURPOSE:	This is a compendium of information about a particular Property.
				
ASSOCIATIONS:	This is owned by a Realtor, and it should be possible to find
                the Realtor from the PropertyImpl object.
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress.server;

import cscie247.distress.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;


public abstract class PropertyImpl extends UnicastRemoteObject
                                    implements Property, java.io.Serializable {
	
	private static int   propertyIDCounter = 0;
	
	private TypeProperty propertyType;
	private User		 objContainer;		// reference to Container object
    private String		 propertyID;		// Assigned by system, read only
                                            //  (called 'tracking number' in 
                                            //  the assignment statement.
    private String		 address;			// number and street, unit number
                                            //  if needed
    private String		 cityStateZip;		// Town, State 5-digit zipcode
    private int			 rooms;			// Bedrooms, kitchen, living room,
                                            //  dining room, den, other
                                            // non-bathroom/non-closet
    private double		 baths;			// Have to allow 1.5 baths
    private int			 parking;			// Number of spaces available
    private	Date		 yearBuilt;		// Month/Day/Year
    private	TypeParking	 parkingType;		// Off-street, not available, garage
    private double		 listingPrice;		// Dollars and cents
    private int			 livingSpace;		// Integral number of square feet in
                                            //  the house
    private	String		 specialFeatures;	// Wax poetic in 25 words or less
    private	String		 description;		// even more lyrical in 256 chars
    private	Date		 dateListed;		// 4-digit year
    private	int			 numberOfFloors;	// Number of floors in property

    /**																		   |
    *
    *	PropertyImpl Constructor.
    *
    */
	public PropertyImpl(TypeProperty mPropertyType,
					User		 mObjContainer,
                    String		 mAddress,
                    String		 mCityStateZip,
                    int			 mRooms,
                    double		 mBaths,
                    int			 mParking,
                    Date		 mYearBuilt,
                    TypeParking	 mParkingType,
                    double		 mListingPrice,
                    int			 mLivingSpace,
                    String		 mSpecialFeatures,
                    String		 mDescription,
                    int			 mNumberOfFloors) throws java.rmi.RemoteException {
    
		propertyType	 = mPropertyType;
		objContainer	 = mObjContainer;
        propertyID		 = generateId();
        address			 = mAddress;
        cityStateZip	 = mCityStateZip;
        rooms			 = mRooms;
        baths			 = mBaths;
        parking			 = mParking;
        yearBuilt		 = mYearBuilt;
        parkingType		 = mParkingType;
        listingPrice	 = mListingPrice;
        livingSpace		 = mLivingSpace;
        specialFeatures	 = mSpecialFeatures;
        description		 = mDescription;
        numberOfFloors	 = mNumberOfFloors;
        
		/// Get when item was listed (now).
		GregorianCalendar mCalendar = new GregorianCalendar();
		dateListed		 = mCalendar.getTime();    	// today	
    }
    
    
	/**																		   |
    * 
    *	Empty method declarations
    *	
    */
	public abstract boolean match(SearchCriteria mCriteria) throws DistressException;
    
    
	/**																		   |
    * 
    *	generate a unique ID for PropertyImpl objects
	*
    *	@return	String		ID in string format
    */
	private static String generateId() {
		
		propertyIDCounter++;
		
		String mID = "" + propertyIDCounter;
		
		for (int i = 6 - mID.length(); i > 0; i--)
			mID = "0" + mID;
		
		mID = "Prop" + mID;
		
		return (mID);
	}
	
    
	/**																		   |
    *
    *	the following small "test" methods are called by compare() to preform
    *	actual comparisons or range test of ints, doubles, strings, Dates
    *	and enumerated types
    *
    */
	protected boolean testInteger(int mVar, Object mMin, Object mMax) {
	
		Integer mTest = new Integer(mVar);
		
		// test for range
		int testMin = ( mTest.compareTo(mMin) );		
		int testMax = ( mTest.compareTo(mMax) );	
        
        boolean mReturn = ( (testMin == +1 && testMax == -1) || 
				              testMin == 0 || testMax == 0 );
		return mReturn;
	}
	protected boolean testDouble(double mVar, Object mMin, Object mMax) {
	
		Double mTest = new Double(mVar);
		
		// test for range
		int testMin = ( mTest.compareTo(mMin) );		
		int testMax = ( mTest.compareTo(mMax) );
        
        boolean mReturn = ( (testMin == +1 && testMax == -1) || 
                              testMin == 0 || testMax == 0);
		return mReturn;
	}
	protected boolean testString(String mVar, Object mVal) {
    
        boolean mReturn = ( mVar.compareTo(mVal) == 0 );
		return mReturn;
	}
	protected boolean testDate(Date mTestDate, Object mMin, Object mMax) {
		// typecast objects
		Date mDateStart = (Date) mMin;
		Date mDateEnd   = (Date) mMax;
		
        boolean mReturn = (mTestDate.after(mDateStart) && mTestDate.before(mDateStart));
		return mReturn;
	}
	protected boolean testEnum(Object mVar, Object mVal) {
        TypeEnum mFirst  = (TypeEnum) mVar;
        TypeEnum mSecond = (TypeEnum) mVal;
        
        boolean mReturn = ( mFirst.getIndex() == mSecond.getIndex() );
		return mReturn;
	}
    
    
	/**																		   |
    * 
    *	Called by match().  takes a string and preforms the appropriate 
    *						comparison
    *	
    *	@param	String		being looked for
	*	@param	Object		Min value in range
	*	@param	Object		Max value in range
    */
    public boolean compare(String mVarName, Object mMin, Object mMax) {
        
        boolean mReturn = true;						// assume match
        
        if (mVarName.equals("propertyType")) {
            if (! testEnum(propertyType, mMin) )
                mReturn = false;
        } else if (mVarName.equals("address")) {
            if (! testString(address, mMin) )
                mReturn = false;
        } else if (mVarName.equals("cityStateZip")) {
            if (! testString(cityStateZip, mMin) )
                mReturn = false;
        } else if (mVarName.equals("rooms")) {
            if (! testInteger(rooms, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("baths")) {
            if (! testDouble(baths, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("parking")) {
            if (! testInteger(parking, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("yearBuilt")) {
            if (! testDate(yearBuilt, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("parkingType")) {
            if (! testEnum(parkingType, mMin) )
                mReturn = false;
        } else if (mVarName.equals("listingPrice")) {
            if (! testDouble(listingPrice, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("livingSpace")) {
            if (! testInteger(livingSpace, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("specialFeatures")) {
            if (! testString(specialFeatures, mMin) )
                mReturn = false;
        } else if (mVarName.equals("description")) {
            if (! testString(description, mMin) )
                mReturn = false;
        } else if (mVarName.equals("numberOfFloors")) {
            if (! testInteger(numberOfFloors, mMin, mMax) )
                mReturn = false;
        }
        
        return mReturn;
    }
    
    
    
	/**																		   |
    * 
    *	Obtains the email of the owner of this listing.
	*
    *	@return	String		the email of the owner who listed this item
    */
    private String obtainEmail() throws java.rmi.RemoteException {
        return objContainer.getEmail();
    }
    
    
	/* 																		   |
	*			Accessors & Mutators
	*/
    
	// avoids instanceof and allows quick dynamic referencing
	public TypeProperty getPropertyType() {							// read only
		return propertyType;
	}
	
	public User getObjContainer() {							
		return objContainer;										// read only
                            // reference to the Realtor who listed this Property
	}
	
	public String getPropertyID() {
        return propertyID;											// read only
    }
    
	public String getAddress() {
        return address;
    }
	public void setAddress(String mAddress) {
        address = mAddress;
    }
    
	public String getCityStateZip() {
        return cityStateZip;
    }
	public void setCityStateZip(String mCityStateZip) {
        cityStateZip = mCityStateZip;
    }
    
	public int getRooms() {
        return rooms;
    }
	public void setRooms(int mRooms) {
        rooms = mRooms;
    }
    
	public double getBaths() {
        return baths;
    }
	public void setBaths(double mBaths) {
        baths = mBaths;
    }
    
	public int getParking() {
        return parking;
    }
	public void setParking(int mParking) {
        parking = mParking;
    }
    
	public Date getYearBuilt() {
        return yearBuilt;
    }
	public void setYearBuilt(Date mYearBuilt) {
        yearBuilt = mYearBuilt;
    }
    
	public TypeParking getParkingType() {
        return parkingType;
    }
	public void setParkingType(TypeParking mParkingType) {
        parkingType = mParkingType;
    }
	public double getListingPrice() {
        return listingPrice;
    }
	public void setListingPrice(double mListingPrice) {
        listingPrice = mListingPrice;
    }
    
	public int getLivingSpace() {
        return livingSpace;
    }
	public void setLivingSpace(int mLivingSpace) {
        livingSpace = mLivingSpace;
    }
    
	public String getSpecialFeatures() {
        return specialFeatures;
    }
	public void setSpecialFeatures(String mSpecialFeatures) {
        specialFeatures = mSpecialFeatures;
    }
    
	public String getDescription() {
        return description;
    }
	public void setDescription(String mDescription) {
        description = mDescription;
    }
    
	public Date getDateListed() {
        return dateListed;
    }
	public void setDateListed(Date mDateListed) {
        dateListed = mDateListed;
    }
    
	public int getNumberOfFloors() {
        return numberOfFloors;
    }
	public void setNumberOfFloors(int mNumberOfFloors) {
        numberOfFloors = mNumberOfFloors;
    } 
 }