/**
*
*	   File:	CourseParser.java
* 
*	 Author:	Russell Lowke
*
*	  Class:	CSCIE-247, Assignment One, Oct 2nd 2003.
*
*	Purpose:	Parses a table of course data into CourseData objects,
*				which are appended to a list.
*
*	 	 Note:	If parser finds non-whitespace data that does not have five (5) tab
*				delimited items then an exception error is thrown.
*				
*				Tab Width = 4
*
*				My variable conventions:
*					p 	stands for property of object
*					m 	stands for "my" and means local to a method
*					Obj stands for an object from this package
*
*/

package cscie247.asn1;

import java.util.*;
import java.io.*;


public class CourseParser {

	private Calendar	pCal;												// calendar obj

	// A holiday list will always be needed to exclude from course dates.
	private ArrayList	pHolidays;											// holiday list
	
	// properties used for building CourseData objects
	private String		pCNumStr;											// course number
	private String		pTitleStr;											// course title
	private Date		pStartDate;											// start date
	private Date		pEndDate;											// end date
	private ArrayList	pOnDaysLst;											// list of days. Sun = 0
	
	
	/**
    *
    *	Constructor: CourseParser 
    *
    *
    *	  This constructor ensures that there is a GregorianCalendar
    *	  available for use.
	*
    */
	public CourseParser() {
		pCal = new GregorianCalendar();
	}
	
	
	/**
    *
    *	setHolidays		(Called by FileReader)
  	*
  	*	  Setter function called from FileReader.
	*
	*
	*	  param mHolidays			Holiday list to be excluded from course dates.
	*
    */
	public void setHolidays(ArrayList mHolidays) {
		pHolidays = mHolidays;
	}
	
	
	/**
    *
    *	parseLine		(Called by FileReader)
  	*
  	*	  Strings recieved from FileReader [mLine] will be in a table format, which
  	*	  is parsed into five forms, an 'On Days List', a course number, a start date, 
  	*	  an end date, and a course title string. Once parsed, table data is stored 
  	*	  as parammeters for this object and buildCourse() is used to create a new
  	*	  CourseData objects which are appended to reference [mCoursesList].
	*
 	*	  Note:	If parser finds non-whitespace data that does not have five (5) tab
 	*			delimited items then an exception error is thrown.
 	*
	*	  param mLine			String of content passed by FileReader.
	*	  param mCoursesList	reference to list of student's courses.
	*
    */
	public void parseLine(String mLine, ArrayList mCoursesList) {
    
		mLine = mLine.trim();													// trim whitespace/junk
	
		// check for five tab delimited values
		StringTokenizer tabItems = new StringTokenizer(mLine, "\t");
		int tokenCount = tabItems.countTokens();
		
		try {
			if (mLine.length() == 0) {			// ignore trimmed junk or "title" line
				// skip
			} else if (tokenCount == 5) {

				String		mOnDaysStr	= tabItems.nextToken();				// get table elements
							pCNumStr	= tabItems.nextToken();
				String		mStartStr	= tabItems.nextToken();
				String		mEndStr		= tabItems.nextToken();
							pTitleStr	= tabItems.nextToken();
		
				pOnDaysLst	= parseDaysToList(mOnDaysStr);
				pStartDate	= parseStrToDate(mStartStr);
				pEndDate	= parseStrToDate(mEndStr);
		
				if (! pTitleStr.equals("title")) {							// ignore label text
				
					// check for valid data
					if (pStartDate == null)
						throw new Exception(mLine + " [BAD START DATE]");
						
					if (pEndDate == null)
						throw new Exception(mLine + " [BAD END DATE]");
					
					if (pOnDaysLst.size() < 1 || pOnDaysLst.size() > 7)
						throw new Exception(mLine + " [BAD DAYS]");
					
					if (pEndDate.before(pStartDate))
						throw new Exception(mLine + " [START DATE after END DATE]");
					
					mCoursesList.add(buildCourse());						// add course to list
											
				}
			} else
				throw new Exception(mLine + " [TOO MANY OR TOO FEW ARGS]");
		}
			
		catch(Exception e) {
			System.out.println("BAD DATA IN COURSES: " + e.getMessage());
		}
    }
	
	
	/**
    *
    *	buildCourse
  	*
  	*	  Uses object parameters assigned by parseLine() to create a new CourseData object.
  	*	  If bad data is encountered then that data is simply omitted.
	*
	*
	*	  return	Returns a new CourseData object.
	*				
    */
	public CourseData buildCourse() {
	
		CourseData mObjReturn = new CourseData(pCNumStr + ": ", pTitleStr);		// make a new CourseData
		
		Date		mTheDate	= pStartDate;									// date holder
		int			mDayOfWeek;													// holds the day of the week
		
		pCal.setTime(mTheDate);													// update calendar to mTheDate
						
		// while mTheDate <= pEndDate
		while( mTheDate.before(pEndDate) || mTheDate.equals(pEndDate) ) {
			
			mDayOfWeek = pCal.get(Calendar.DAY_OF_WEEK) - 1;					// get the day of the week
			
			if (isDateOnDaysList(mDayOfWeek) &&
			  ! isDateAHoliday(mTheDate))
				mObjReturn.addDate(mTheDate);									// add date to new CourseData
			
			pCal.add(Calendar.DATE, 1);											// go to next calendar day
			mTheDate = pCal.getTime();											// update mTheDate
		}
		return mObjReturn;		
	}
	

	/**
    *
    *	isDateOnDaysList
  	*
  	*	  Returns true if [mDayOfWeek] is in the list [pOnDaysLst] .
	*
	*
	*	  param		mDayOfWeek	Day of week being tested for, 0 = Sunday.
	*
	*	  return				true, if [mDayOfWeek] is in the parameter list pOnDaysLst
	*				
    */
	public boolean isDateOnDaysList(int mDayOfWeek) {
	
		boolean mReturn = false;
		
		// is date is in pOnDaysLst list?
		Iterator e = pOnDaysLst.iterator();
		while(e.hasNext() && ! mReturn) {
			if ( stringToInt((String) e.next()) == mDayOfWeek)
				mReturn = true;
		}
		
		return 	mReturn;	
	}
	

	/**
    *
    *	isDateAHoliday
  	*
  	*	  Returns true if Date [mTheDate] is in the holiday list [pHolidays].
	*
	*
	*	  param		mTheDate	Day of week being tested for, 0 = Sunday.
	*
	*	  return				true, if [mTheDate] is in the holiday list [pHolidays].
	*				
    */
	public boolean isDateAHoliday(Date mTheDate) {
	
		boolean mReturn = false;
	
		// is date a holiday?
		Iterator e = pHolidays.iterator();
		while(e.hasNext() && ! mReturn) {
			if (mTheDate.equals( (Date) e.next()))
				mReturn = true;
		}
		
		return mReturn;	
	}
	
	
	/**
    *
    *	parseDaysToList
  	*
  	*	  Converts the string [mOnDaysStr] which should be in format "Mon:Tues:Wed",
  	*	  into an ArrayList list in format ["1","2","3"], where Sunday = 0.
  	*	
  	*	  Note: ArrayList does not like integers, so I stored the numbers as
  	*			strings, and must convert them using stringToInt() when required.
	*
	*
	*	  param		mOnDaysStr	String in "Mon:Tues:Wed" format.
	*
	*	  return				corresponding ArrayList list in format ["1","2","3"].
	*				
    */
	public ArrayList parseDaysToList(String mOnDaysStr) {
	
		ArrayList mReturn = new ArrayList();
		StringTokenizer dayDayDay = new StringTokenizer(mOnDaysStr, ":");
		String mDay;
		
		while(dayDayDay.hasMoreTokens()) {
		
			mDay = dayDayDay.nextToken();										// get the next token
			mDay.trim();														// trim out junk
			mDay = mDay.substring(0, 2);										// look for first two chars
			
			if (mDay.equalsIgnoreCase("Su")) {
				mReturn.add("0");
			} else if (mDay.equalsIgnoreCase("Mo")) {
				mReturn.add("1");
			} else if (mDay.equalsIgnoreCase("Tu")) {
				mReturn.add("2");
			} else if (mDay.equalsIgnoreCase("We")) {
				mReturn.add("3");
			} else if (mDay.equalsIgnoreCase("Th")) {
				mReturn.add("4");
			} else if (mDay.equalsIgnoreCase("Fr")) {
				mReturn.add("5");
			} else if (mDay.equalsIgnoreCase("Sa")) {
				mReturn.add("6");
			}
		}
			
		return mReturn;
		
	}
	
	
	/**
    *
    *	parseStrToDate
  	*
  	*	  Converts a string of form "mm/dd/yy" into a Date object.
  	*	  Unsucessful attempts result in null.
  	*
  	*
  	*	  param mDateStr	The date string in neeed of conversion.
  	*
  	*	  return			A Date object, or null
	*
    */
	public Date parseStrToDate(String mDateStr) {
	
		Date mReturn = null;													// default return
		
		// ensure three values
		StringTokenizer mmddyyyy = new StringTokenizer(mDateStr, "/");
		if (mmddyyyy.countTokens() == 3) {
		
			int mMonth = stringToInt(mmddyyyy.nextToken());		
			int mDay   = stringToInt(mmddyyyy.nextToken());
			int mYear  = stringToInt(mmddyyyy.nextToken());
	
			// ensure valid month and day values
			if (mMonth >= 1 && mMonth <= 12 &&
				mDay   >= 1 && mDay   <= 31) {
				
				pCal = new GregorianCalendar(mYear, mMonth - 1, mDay);			// month is from 0 to 11
				mReturn = pCal.getTime();
			}
		}
	
		return mReturn;
	}
	

	/**
    *
    *	stringToInt
    *
    *	  Converts a string to an integer.
  	*
  	*
  	*	  param mString		The string to convert.
  	*
  	*	  return			An integer representation of the given string.
	*
    */
	private static int stringToInt(String stringObject) {
		return Integer.parseInt(stringObject.trim());
	}
}