/*******************************************************************************
 
 
 	    FILE:	Distress.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Assignment Five

 
 	 PURPOSE:	Distress is the focal point of interaction with external 
                objects.  It has three main functions: providing the ability for
                new users to gain access to the system, allowing existing users 
                to login to the system, and supporting ad hoc searches for 
                properties or mortgages.   There should be only one Distress 
                object instance.
				
ASSOCIATIONS:	The Distress object keeps a collection of User objects.
 	
	MODIFIED:	December 14th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					f 	= "field", indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn5.server;

import cscie247.asn5.*;
import java.util.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;


public class DistressImpl extends UnicastRemoteObject
                                implements Distress, java.io.Serializable {

    private static Distress fDistress = null;				// singleton
    private static ArrayList fUsers = new ArrayList();      // User objects
                                                            // in Distress
    
    // Don't let anyone else instantiate this class   (singleton)
	private DistressImpl() throws RemoteException {
        super();			// invokes no-arg constructor of UnicastRemoteObject
                            // which "exports" it by listening for incoming calls
    }
    
	public static Distress getDistress() throws RemoteException {  // read only
        
        // ensure Distress is a singleton.  Only instantiate once.
        if (fDistress == null) {
            fDistress = new DistressImpl();
        }
		return fDistress;
	}
    
    
	///																		   |
    // 
    //	This method takes all the attributes given and causes a new User to be 
    //	insterted in the system.  TypeUser is an enumeration of the kinds of 
    //	users (Realtor, Consumer, Lender).  It returns a reference to the new 
    //	user.  This throws an exception if loginName is already in use.
    //
	//	@param	TypeUser	The type of user registering
	//	@param	String		name of the user
	//	@param	String		address of user
	//	@param	String		city state and zipcode
	//	@param	String		user login name
	//	@param	String		user pin
	//	@param	String		user email
	//	@param	String		user credit card company
	//	@param	String		user credit card number
	//	@param	Object		object specific to the userType [mUt]
	//	@return	User		Object reference to the new User
    //

	public User register (TypeUser  mUserType,
						  String    mName,
						  String	mAddress,
						  String	mCityStateZip,
						  String	mLoginName,
						  String	mPin,
						  String	mEmail,
						  String	mCreditCardCo,
						  String	mCreditCardNumber,
						  Object	mObject) throws ErrDistress, ErrRegistration, java.rmi.RemoteException {
		
		User mUser = null;
		
        // check for any Users with the same login name
		if ( getUser(mLoginName) != null )
			throw new ErrRegistration("A user already exists with a loginName of \"" 
								+ mLoginName + "\"\n");
		
		if (mUserType == TypeUser.CONSUMER) {
		
			// convert the mObject
			TypePurchasingTF mPurchasingTimeFrame = 
							(TypePurchasingTF) mObject;
			
			mUser = new ConsumerImpl(mUserType,
								 mName,
								 mAddress,
								 mCityStateZip,
								 mLoginName,
								 mPin, 
								 mEmail, 
								 mCreditCardCo, 
								 mCreditCardNumber,
								 mPurchasingTimeFrame);
					
		} else if (mUserType == TypeUser.REALTOR) {			
		
			// convert the mObject
			String mAgency = (String) mObject;
			
			mUser = new RealtorImpl(mUserType,
								mName,
								mAddress,
								mCityStateZip,
								mLoginName,
								mPin, 
								mEmail, 
								mCreditCardCo, 
								mCreditCardNumber,
								mAgency);
		}
		
		if (mUser != null)
			add(mUser);
		
		return mUser;
	}
    
    
	///																		   |
    // 
    //	Causes this User to be removed from the system along with all their 
    //	appendages (for instance, when a Realtor is unregistered, it is 
    //	ultimately deleted from the system along with all Property objects it 
    //	owns).  Throws an exception if loginName doesn't exist.
    //
	//	@param	String	login name of the user to be removed
    //

    public void unregister(String mLoginName) 
                    throws ErrDistress, ErrRegistration, java.rmi.RemoteException {
		remove(getUser(mLoginName));
    }
    

	///																		   |
    // 
    //	Validates credentials and creates an object for a user.  Throws an
    //	exception if the credentials are invalid.
    //
	//	@param	String	name of the user
    //	@param	String	password entered by user
	//	@return	User	User object to that person
    //

    public User login(String mLoginName, String mPassword) 
                    throws ErrDistress, ErrRegistration, java.rmi.RemoteException {

        User mUser = getUser(mLoginName);
        if (mUser == null)
            throw new ErrRegistration("Could not find user with login of \n" + 
                                    mLoginName + "\n");
        
        if ( ! (mUser.getPin()).equals(mPassword) )
            throw new ErrRegistration("Pin/Password Incorect.");
                
        return mUser;
    }


	///																		   |
    // 
    //	Returns a collection (possibly empty) of the Property objects that match
    //	the critera.  In searching for a Property, the proprty type and city 
    //	must be given.
    //
	//	@param	SearchCriteria	object which defines what to search for
	//	@return	ArrayList	 	list of Property objects that match the search
    //

    public ArrayList findProperty(SearchCriteria mSearchCriteria)
                        throws ErrDistress, java.rmi.RemoteException {
	
		if ( ! mSearchCriteria.getHasPropertyTypeAndCity() ) {
			throw new ErrDistress("SearchCriteria requires a propertyType" +
									" and cityStateZip to find a property");
		}
		
        ArrayList mReturn = new ArrayList();
        Property mProperty;
		
		// iterate through properties looking for search critera
		Iterator mItr = listProperties();
		while(mItr.hasNext()) {
			mProperty = (Property) mItr.next();
			
			// add any properties that match search criteria to mReturn
			if ( mProperty.match(mSearchCriteria) )
				mReturn.add(mProperty);
		}
                
        return mReturn;
    }


	///																		   |
    // 
    //	Returns the User object which has a specific loginID  [mLoginID]
	//
    //	@parame	String			the specific loginID
    //	@return	Iterator		Iterator to pMortgageOfferings.
    //

	public User getUser(String mLoginID) throws java.rmi.RemoteException {
	
		User	 mReturn = null;
		User     mUser;
		Iterator mItr = list();
		
		while(mItr.hasNext() && mReturn == null) {
			mUser = (User) mItr.next();
			if ( (mUser.getLoginName()).equals(mLoginID) )
				mReturn = mUser;
		}
		
		return mReturn;
	}
    

	///																		   |
    // 
    //	Adds the User instance mUser to the fUsers list, if not already there
	//
    //	@param	User		the User instance to be added to fUsers list.
    //

    public void add(User mUser)
                    throws ErrDistress, ErrRegistration, java.rmi.RemoteException {
    
        // check for other users with the same login name.
        User mOtherUser = getUser(  mUser.getLoginName()  );
		if (mOtherUser != null)
          throw new ErrRegistration("There is already a user with a login of \n"
									+ mUser.getLoginName() + "\n");
                                    
                                    
        // Note: error checking for any Users with the same login name
        // is done in register()

        Iterator mItr = findUser(mUser);
        
        if (mItr == null)
            fUsers.add(mUser);
        else
			throw new ErrRegistration("CAN'T ADD USER. \"" + mUser.getName() +
								  "\" is already in User List.");
			
    }
    

	///																		   |
    // 
    //	Removes the User instance mUser from the fUsers list, if found.
	//
    //	@param	User		the User instance to be removed from fUsers list.
    //

    public void remove(User mUser)
                    throws ErrRegistration, java.rmi.RemoteException {
    
        Iterator mItr = findUser(mUser);
        
        if (mItr == null)
			throw new ErrRegistration("CAN'T REMOVE USER. \"" + mUser.getName() +
								  "\" is not in User List.");
        else
            mItr.remove();
    }
    

	///																		   |
    // 
    //	Returns an iterator to the fUsers list
	//
    //	@return	String		a summary of pertinent values.
    //

    public Iterator list() {
        return  fUsers.iterator();
    }
    

	///																		   |
    // 
    //	Searches through the fUsers list looking for a User instance with the
    //	same reference as mUser.  If found, an Iterator is returned resting on
    //	the list item (so it may be removed easily), 
    //	otherwise a null is returned.
	//
    //	@return	Iterator	points to item mUser in fUsers. if mUser can't be
    //						found then returns null
    //

	private Iterator findUser(User mUser) {
    
        boolean mFoundIt = false;
        Iterator mItr = list();
		while(mItr.hasNext() && ! mFoundIt)
            if (mUser == mItr.next())
                mFoundIt = true;
                
        if (! mFoundIt)
            mItr = null;
        
        return mItr;
    }

    
	///																		   |
    // 
    //	Helper methods that invoke listHeldByUser(), thus returning an Iterator 
    //	a list of the requested objects
    //	
    //	@return	Iterator	to list of items held by Users of approp. type
    //

    public Iterator listProperties() 
                        throws java.rmi.RemoteException {
		return listHeldByUser(TypeUser.REALTOR);
	}
	public Iterator listMortgageOfferings() 
                        throws java.rmi.RemoteException {
		return listHeldByUser(TypeUser.LENDER);
	}
	public Iterator listPropertyProfiles() 
                        throws java.rmi.RemoteException {
		return listHeldByUser(TypeUser.CONSUMER);
	}
    
    
	///																		   |
    // 
    //	Helper method that returns an Iterator to a list of all
    //	Users of the type [mUserType]
    //	
    //	@param	TypeUser	type of User targeted
    //	@return	Iterator	list of objects held by that User
    //

	public Iterator listHeldByUser(TypeUser mUserType)
                        throws java.rmi.RemoteException {
                        
		ArrayList mReturn = new ArrayList();
		
		// iterate through Users
		Iterator mItr = list();
		User	 mUser;
		Iterator mItr2;
		
		while(mItr.hasNext()) {
			mUser = (User) mItr.next();
			
			// find Realtors
			if (mUser.getUserType() == mUserType) {
				
				// add Realtors Properties to mReturn
				mItr2 = mUser.list();
				while(mItr2.hasNext())
					mReturn.add(mItr2.next());
			}
		}

		return mReturn.iterator();
	}

	
	public ArrayList getUsers() {
		return fUsers;
	}
	public void setUsers(ArrayList mUsers) {
		fUsers = mUsers;
	}
}