/*************************************************
 *
 * Adapted from Java Examples In A Nutshell
 * by David Flanagan
 * Copyright 1997
 *************************************************/

package rmiexample;

import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Date;

public class RemoteBankImpl extends UnicastRemoteObject implements RemoteBank {

   /** This is a nested class **/
 /**
  * The account class stores data for a single
  *  account with the bank.
  */
  class Account {
      String password;         // account password;
      int balance;             // account balance;
      Vector transactions = new Vector(); // account transaction history
      Account(String password) {
        this.password = password;
        transactions.addElement("Account opened at " + new Date());
      }
    }

    /**
     * This hashtable stores all open accounts and maps from account name
     * to account object
     **/
     Hashtable accounts = new Hashtable();

     /**
      * This constructor doesn't do anything, but because it throws the
      * same exception that the superclass constructor throws, it must
      * be declared.
      **/
      public RemoteBankImpl() throws RemoteException {
        super();
        System.out.println("Remote Bank Impl created.");
      }

      /**
       * Open a bank account with the specified name and password
       * This method is synchronized to make it thread safe, since it
       * manipulates the accounts hashtable.
       **/
       public synchronized void openAccount(String name, String password)
        throws RemoteException, BankingException
      {
        // Check if there is already an account under that name
        if(accounts.get(name) != null)
          throw new BankingException("Account already exists.");
        // Otherwise it doesn't exist, so create it.
        Account acct = new Account(password);
        // And register it.
        accounts.put(name, acct);
      }

      /**
       * This convenience method is not a remote method. Given a name and
       * password, it checks to see if an account with that name and
       * password exists. If so, it returns the Account object. Otherwise
       * it throws an exception.
       **/
       public Account verify(String name, String password) throws BankingException
       {
         synchronized(accounts) {
           Account acct = (Account) accounts.get(name);
           if(acct == null)
             throw new BankingException("No such account.");
           if(! password.equals(acct.password))
             throw new BankingException("Invalid password");
           return(acct);
          }
       }

       /**
        * Close the named account. This method is synchronized to make it
        * thread safe, since it manipulates the accounts hashtable.
        **/

       public synchronized FunnyMoney closeAccount(String name, String password)
        throws RemoteException, BankingException
        {
          Account acct;
          acct = verify(name, password);
          accounts.remove(name);
          // Before changing the balance or transactions of any account, we first
          // have to obtain a lock on that account to be thread safe.
          synchronized(acct) {
            int balance = acct.balance;
            acct.balance = 0;
            return(new FunnyMoney(balance));
          }
        }

        /**
         *   Deposit the specified FunnyMoney to the named account
         */
         public void deposit(String name, String password, FunnyMoney money)
           throws RemoteException, BankingException
           {
              Account acct = verify(name, password);
              synchronized(acct) {
                acct.balance += money.amount;
                acct.transactions.addElement("Deposited " + money.amount +
                  " on " + new Date());
              }
           }

         /**
          * Withdraw the specified amount from the named account
          */
           public FunnyMoney withdraw(String name, String password, int amount)
              throws RemoteException, BankingException
           {
             Account acct = verify(name, password);
             synchronized(acct) {
               if(acct.balance < amount)
                  throw new BankingException("Insufficient Funds");
               acct.balance -= amount;
               acct.transactions.addElement("Withdraw " + amount + " on " + new Date());
               return new FunnyMoney(amount);
              }
            }

         /**
          * Return the current balance in the named account
          */
          public int getBalance(String name, String password) throws RemoteException,
            BankingException
          {
              Account acct = verify(name, password);
              synchronized(acct) {
                  return(acct.balance);
              }
          }

          /**
           * Return a Vector of strings containing the transaction history
           * for the named account
           */
           public Vector getTransactionHistory(String name, String password)
              throws RemoteException, BankingException
           {
              Account acct = verify(name, password);
              synchronized(acct) {
                  return(acct.transactions);
              }
           }

}