/*******************************************************************************
 
 
 	    FILE:	RealtorScreen.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project
 
 
 	 PURPOSE:	RealtorScreen.  The infoPanel has a welcome message for the 
				realtor who just  logged in.  The workingPanel shows a list of
				Property objects (id, date listed,  address, asking price) posted
				by this realtor.  The commandPanel has buttons  'Edit' and 'Close'.
				When the user clicks on a property to select it and then clicks on
				the 'Edit' button, the next screen is the SingleFamilyEditScreen.
				Clicking on  'Close' logs the realtor out, closes the RealtorScreen,
				and returns to the  MainDistressScreen.
				
ASSOCIATIONS:	Called by MainDistressScreen
				"Edit" btn opens SingleFamilyEditScreen
				"Close" btn opens MainDistressScreen
				Child of abstract class DistressScreen
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/


package cscie247.distress.client;

import cscie247.distress.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class RealtorScreen extends DistressScreen {
    
	public static RealtorScreen	thisWindow = null;
    public static Realtor		user;
    
    private JList				list;
    private DefaultListModel	listModel;
    
    int	    fLastSelected = 0;
	private ArrayList			properties;
    
	private JButton fEditBtn;
	private JButton fCloseBtn;


	/**																		   |
    *
    *	main allows this screen to be displayed independent of Distress,
    *	which is useful for testing, and saves me having to fireup
    *	the Distress server if I'm not on a network
    */
	public static void main(String args[]) {
    
        distress = null;			// Distress not loaded
        openWindow();     
	}
    
    
	public RealtorScreen() {
        super("Realtor");
	}
    
    public static void openWindow() {
        openWindow(user);			// use existing user
    }
    
	/**																		   |
    * 
    *	Static method that goes about opening this window and adding 
    *	a window listener to conclude Distress if this window is closed
    *	The MDSWindowListener is found in the abstract class DistressScreen.java
    *	Note: thisWindow is a static field that contains this window's object
    *	
    *	@param	Realtor		The Realtor object being viewed
    */
	public static void openWindow(Realtor mUser) {
    
        user = mUser;
        
        // instance of this window
        thisWindow = new RealtorScreen();
        thisWindow.addWindowListener(thisWindow.new MDSWindowListener());
        thisWindow.setSize(400, 200);
        positionWindow( (DistressScreen) thisWindow );
        thisWindow.setVisible(true);
    }
    
    
	/**																		   |
    *																   
    *	static method to hide this window and dispose of it.
    *
    */
    private static void closeWindow() {
        thisWindow.setVisible(false);
        thisWindow = null;
    }
    
	
	/**																		   |
    *
    *	Sets up the information panel (template pattern)
    *   The infoPanel has a welcome message for the realtor who just logged in.
	*
    *	@return	JPanel		JPanel using a BorderLayout
    */
	public JPanel setUpInfoPanel() {
        
        JPanel mReturn = new JPanel();
        mReturn.setLayout(new BorderLayout());
        
        // load vars from Distress (if loaded)
        String mUserName = "";
        if (user != null) {
            try {
                mUserName = user.getName();
            } catch (java.rmi.RemoteException e) {
                alert(e.getMessage());
            }
        }
        
        // setup welcome Realtor screen
        mReturn.add(new JLabel("Welcome "+ mUserName + " [ Realtor ]",
                             JLabel.CENTER), BorderLayout.NORTH);
        mReturn.add(new JLabel("Select the property you wish to edit",
                             JLabel.CENTER),BorderLayout.CENTER);

        
        return mReturn;
	}


	/**																		   |
    *
    *	Sets up the working panel (template pattern)
	*   The workingPanel shows a list of Property objects (id, date listed,
	*   address, asking price) posted by this realtor.
    *
    *	@return	JPanel		JPanel using a GridBagLayout
    */
	public JPanel setUpWorkingPanel() {
        GridBagLayout gbLayout = new GridBagLayout();
        JPanel mReturn = new JPanel();
        //mReturn.setLayout(new BorderLayout());
        
        try {
        
            //Create the list and put it in a scroll pane
            listModel = new DefaultListModel();
            
            if (user != null) {
                properties   = (ArrayList) user.getProperties();
                Iterator mItr = properties.iterator();
                
                Property mObj;
                while(mItr.hasNext()) {
                    mObj = (Property) mItr.next();
                    listModel.addElement( mObj.getPropertyID() + "  " +
                                    dateFormatmmddyyyy(mObj.getDateListed()) + "  " +
                                    mObj.getAddress() + "  " + 
                                    "$" + mObj.getListingPrice());
                }
            }
            
            list = new JList(listModel);
            list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            list.setSelectedIndex(0);
            list.addListSelectionListener(new SelListener());
            JScrollPane mListScrollPane = new JScrollPane(list);
            
            mReturn.add(mListScrollPane);
        
        } catch (java.rmi.RemoteException e) {
            alert(e.getMessage());
        }
        
        return mReturn;
	}
    
  
	/**																		   |
    *
    *	Sets up the command panel (template pattern)
	*   The commandPanel has buttons  'Edit' and 'Close'.
	*   When the user clicks on a property to select it and then clicks on
	*   the 'Edit' button, the next screen is the SingleFamilyEditScreen.
	*   Clicking on  'Close' logs the realtor out, closes the RealtorScreen,
	*   and returns to the  MainDistressScreen.
    *
    *	@return	JPanel		JPanel using a default layout
    */
	public JPanel setUpCommandPanel() {
    
        /*
        * Set up the JButton for Close
        */
        fCloseBtn = new JButton("Close");
        fCloseBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closeWindow();
                MainDistressScreen.openWindow();
            }
        });
        
        /*
        * Set up the JButton for Close
        */
        fEditBtn  = new JButton("Edit");
        fEditBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                RealtorScreen.closeWindow();
                SingleFamilyEditScreen.openWindow( (SingleFamily) properties.get(fLastSelected));
            }
        });
        
        JPanel mReturn = new JPanel();
        
        mReturn.add(fEditBtn);
        mReturn.add(new JLabel("     ")); // a spacer
        mReturn.add(fCloseBtn);
        
        return mReturn;
	}
    
	
    /**
	*
	*   SelListener listens for changes to an edit field and enables or 
	*   disables the Edit button accordingly.
	*/
    private class SelListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            
            if (e.getValueIsAdjusting() == false) {
                int mSelected = list.getSelectedIndex();
                
                try {
                    //Selection, update text field.
                    Property mPropertySelected = (Property) properties.get(mSelected);
                    
                    // only enable edit button for SINGLEFAMILY properties
                    if (mPropertySelected.getPropertyType().toString().equals("SINGLEFAMILY"))
                        fEditBtn.setEnabled(true);
                    else 
                        fEditBtn.setEnabled(false);
                        
                    fLastSelected = mSelected;
                } catch (java.rmi.RemoteException em) {
                    alert(em.getMessage());
                }
            }
        }
    }
}